#pragma once

#include <stdint.h>
#include "i2c/memory.hpp"


namespace Devices
{
    template <class I2cImpl>
    class Ds3231
    {
        private:

            using Base = I2c::Memory<I2cImpl>;

        public:

            static uint8_t const ADDRESS = 0xd0;

            static uint8_t const REG_CONTROL = 0x0e;
            static uint8_t const EOSC = 7;
            static uint8_t const BBSQW = 6;
            static uint8_t const CONV = 5;
            static uint8_t const RS2 = 4;
            static uint8_t const RS1 = 3;
            static uint8_t const INTCN = 2;
            static uint8_t const A2IE = 1;
            static uint8_t const A1IE = 0;
            
            static uint8_t const REG_STATUS = 0x0f;
            static uint8_t const OSF = 7;
            static uint8_t const EN32KHZ = 3;
            static uint8_t const BSY = 2;
            static uint8_t const A2F = 1;
            static uint8_t const A1F = 0;
            
            static uint8_t const REG_AGING_OFFSET = 0x10;
            static uint8_t const REG_TEMPERATURE_H = 0x11;
            static uint8_t const REG_TEMPERATURE_L = 0x12;

            enum class SquareWave 
            {
                Wave1Hz = 0,
                Wave1024Hz = _BV(RS1),
                Wave4096Hz = _BV(RS2),
                Wave8192Hz = _BV(RS2) | _BV(RS1),
                WaveDisabled
            };

            static bool Init()
            {
                Base::Init();
                return true;
            }

            static bool SetSquareWave(SquareWave const squareWave)
            {
                uint8_t control;

                if (!Base::Read(ADDRESS, REG_CONTROL, control))
                {
                    return false;
                }
                
                if (squareWave == SquareWave::WaveDisabled)
                {
                    control |= _BV(INTCN);
                }
                else
                {
                    control &= ~(_BV(INTCN) | _BV(RS2) | _BV(RS1));
                    control |= (uint8_t) squareWave;
                }

                return Base::Write(ADDRESS, REG_CONTROL, control);
            };

            static bool Set32kHz(bool enabled)
            {
                uint8_t status;

                if (!Base::Read(ADDRESS, REG_STATUS, status))
                {
                    return false;
                }
                
                if (enabled)
                {
                    status |= _BV(EN32KHZ);
                }
                else
                {
                    status &= ~_BV(EN32KHZ);
                }

                return Base::Write(ADDRESS, REG_STATUS, status);
            };
    };
}