#pragma once

namespace Devices::Mcu
{
    template <class PortRegistersT, uint8_t pinNumber>
    struct Pin
    {

        static void ConfigureAsInput()
        {
            *(PortRegistersT::DDR()) &= ~(1 << pinNumber);
        }


        static void ConfigurePullUp(bool pullUp)
        {
            if (pullUp)
            {
                *(PortRegistersT::PORT()) |= (1 << pinNumber);
            }
            else
            {
                *(PortRegistersT::PORT()) &= ~(1 << pinNumber);
            }
        }


        static void ConfigureAsInput(bool pullUp)
        {
            ConfigureAsInput();
            ConfigurePullUp(pullUp);
        }


        static void ConfigureAsOutput()
        {
            *(PortRegistersT::DDR()) |= (1 << pinNumber);
        }

        static void SetHigh()
        {
            *(PortRegistersT::PORT()) |= (1 << pinNumber);
        }

        static void SetLow()
        {
            *(PortRegistersT::PORT()) &= ~(1 << pinNumber);
        }

        static void Set(bool value)
        {
            if (value)
            {
                SetHigh();
            }
            else
            {
                SetLow();
            }
        }

        static bool Get()
        {
            return *(PortRegistersT::PIN()) & (1 << pinNumber);
        }
    };

    template <class PortRegistersT>
    struct Port
    {
        using Pin0 = Pin<PortRegistersT, 0>;
        using Pin1 = Pin<PortRegistersT, 1>;
        using Pin2 = Pin<PortRegistersT, 2>;
        using Pin3 = Pin<PortRegistersT, 3>;
        using Pin4 = Pin<PortRegistersT, 4>;
        using Pin5 = Pin<PortRegistersT, 5>;
        using Pin6 = Pin<PortRegistersT, 6>;
        using Pin7 = Pin<PortRegistersT, 7>;

        static void Init()
        {
        }

        static void SetDirection(uint8_t direction)
        {
            *(PortRegistersT::DDR()) = direction;
        }

        static void Out(uint8_t data)
        {
            *(PortRegistersT::PORT()) = data;
        }

        static void In(uint8_t & data)
        {
            data = *(PortRegistersT::PIN());
        }
    };
}