#pragma once

#include <stdint.h>
#include "../utils/int_least.hpp"


namespace Devices
{
    template<class SerialOutT, class SerialClkT, class LatchClkT, uint8_t N>
    class Ttl74595
    {
        public:

            using DataT = Utils::UIntLeast<N * 8>;

            static void Init()
            {
                SerialOutT::ConfigureAsOutput();
                SerialOutT::SetLow();
                SerialClkT::ConfigureAsOutput();
                SerialClkT::SetLow();
                LatchClkT::ConfigureAsOutput();
                LatchClkT::SetLow();
            }

            static void Out(DataT const data)
            {
                DataT b = data;
                constexpr DataT mask = (DataT) 1 << (N * 8 - 1);

                for (uint8_t i = 0; i < N * 8; ++i)
                {
                    SerialOutT::Set((b & mask) != 0);
                    SerialClkT::SetHigh();
                    SerialClkT::SetLow();
                    b <<= 1;
                }

                SerialOutT::SetLow();
                LatchClkT::SetHigh();
                LatchClkT::SetLow();
            }

            static DataT In()
            {
                return 0;
            }
    };
}
